package org.coderu.adapters.junit;

import static org.coderu.common.utils.CollectionUtils.transform;

import java.util.Collection;

import junit.framework.Assert;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.coderu.core.api.ClazzDependency;
import org.coderu.core.api.CompileUnit;
import org.coderu.core.api.DependencyOnSameLevelAllowed;
import org.coderu.core.api.Depth;
import org.coderu.core.api.Packagge;
import org.coderu.core.api.WrongDependencyExplorer;

import com.google.common.base.Function;


public final class CoderuAssert {

    public void assertNoViolations(
        final Collection<CompileUnit> unit, 
        final Collection<Packagge> package2test, 
        final Collection<Packagge> package2excude){

        assertNoViolations(unit, package2test, package2excude, Depth.UNLIMITED);
    }

    public void assertNoViolations(
        final Collection<CompileUnit> unit, 
        final Collection<Packagge> package2test, 
        final Collection<Packagge> package2excude,
        final Depth depth){
        
        final Collection<ClazzDependency> notAllowedDependencies = 
            dependencyExplorer.calculate(unit, package2test, package2excude, depth, DependencyOnSameLevelAllowed.YES);

        assertEmpty(notAllowedDependencies);
    }

    public CoderuAssert(WrongDependencyExplorer dependencyExplorer) {
        this.dependencyExplorer = dependencyExplorer;
    }

    private final WrongDependencyExplorer dependencyExplorer;

    private static void assertEmpty(final Collection<ClazzDependency> notAllowedDependencies) {

        final Collection<String> strings = 
            transform(notAllowedDependencies, ACCESS_RIGHT_2_STRING);

        Assert.assertEquals("not allowed dependencies exist","", 
            StringUtils.join(strings,IOUtils.LINE_SEPARATOR_UNIX));
    }

    private static final Function<ClazzDependency,String> ACCESS_RIGHT_2_STRING = 
        new Function<ClazzDependency, String>() {
        public String apply(ClazzDependency o) {
            return String.format("%s => %s", o.getFirst().getName(), o.getSecond().getName());
        }
    };

}